import yaml
import click

from .errors import InvalidAutoscaleMinError, InvalidInstanceError


def validate_jcloud_config(config_path):
    with open(config_path, "r") as f:
        config_data = yaml.safe_load(f)
        instance = config_data.get("instance")
        autoscale_min = config_data.get("autoscale_min")

        if instance and not (instance.startswith("C") and instance[1:].isdigit()):
            raise InvalidInstanceError(instance)

        if autoscale_min:
            try:
                autoscale_min_int = int(autoscale_min)
                if autoscale_min_int < 0:
                    raise InvalidAutoscaleMinError(autoscale_min)
            except ValueError:
                raise InvalidAutoscaleMinError(autoscale_min)


def validate_jcloud_config_callback(ctx, param, value):
    if not value:
        return None
    try:
        validate_jcloud_config(value)
    except InvalidInstanceError as e:
        raise click.BadParameter(
            f"Invalid instance '{e.instance}' found in config file', please refer to https://docs.jina.ai/concepts/jcloud/configuration/#cpu-tiers for instance definition."
        )
    except InvalidAutoscaleMinError as e:
        raise click.BadParameter(
            f"Invalid instance '{e.min}' found in config file', it should be a number >= 0."
        )

    return value



import asyncio
from typing import Dict

import nest_asyncio
from lcserve import serving, download_df
from fastapi import WebSocket

from pandasai import PandasAI
from pandasai.llm.openai import OpenAI

nest_asyncio.apply()

EXIT_PROMPT = "exit"


class ConversationalPandasAI(PandasAI):
    def __init__(self, websocket: WebSocket, **kwargs):
        self.websocket = websocket
        super().__init__(**kwargs)

    def log(self, message: str):
        super().log(message)
        asyncio.run(self.websocket.send_json({"message": message}))


@serving
def ask(url: str, prompt: str, **kwargs) -> str:
    df = download_df(url)
    llm = OpenAI()
    pandas_ai = PandasAI(llm, verbose=True)
    return pandas_ai.run(df, prompt=prompt)


@serving(websocket=True)
async def converse(url: str, **kwargs) -> str:
    websocket: WebSocket = kwargs.get("websocket")
    try:
        df = download_df(url)
    except Exception as e:
        await websocket.send_json({"message": f"Error downloading dataframe: {e}"})
        return

    await websocket.send_json(
        {"message": f"Downloaded dataframe with shape {df.shape}. Send me a prompt!"}
    )

    while True:
        prompt_input: Dict = await websocket.receive_json()
        prompt = prompt_input.get("prompt")
        if prompt is None or prompt.lower() == EXIT_PROMPT:
            return "Bye!"
        llm_args = prompt_input.get("llm_args", {})
        llm = OpenAI(**llm_args)
        pandas_ai = ConversationalPandasAI(websocket=websocket, llm=llm, verbose=True)
        answer = pandas_ai.run(df, prompt=prompt)
        await websocket.send_json({"answer": answer})
        await websocket.send_json(
            {"message": 'Waiting for next prompt, OR pass "exit" to exit.'}
        )



import platform
import sys
from os import path

import pkg_resources
from setuptools import find_packages, setup


def get_requirements_list(f):
    with pathlib.Path(f).open() as requirements_txt:
        requirements_list = [
            str(requirement)
            for requirement in pkg_resources.parse_requirements(requirements_txt)
        ]
        return requirements_list


if sys.version_info < (3, 7, 0):
    raise OSError(f'langchain-serve requires Python >=3.7, but yours is {sys.version}')

try:
    pkg_name = 'langchain-serve'
    libinfo_py = path.join('lcserve', '__init__.py')
    libinfo_content = open(libinfo_py, 'r', encoding='utf8').readlines()
    version_line = [l.strip() for l in libinfo_content if l.startswith('__version__')][
        0
    ]
    exec(version_line)  # gives __version__
except FileNotFoundError:
    __version__ = '0.0.0'

try:
    with open('README.md', encoding='utf8') as fp:
        _long_description = fp.read()
except FileNotFoundError:
    _long_description = ''

import os
import pathlib

install_requires = get_requirements_list(
    os.path.join(os.path.dirname(__file__), 'requirements.txt')
)
sys_platform = platform.system().lower()


setup(
    name=pkg_name,
    packages=find_packages(),
    version=__version__,
    include_package_data=True,
    description='Langchain Serve - serve your langchain apps on Jina AI Cloud.',
    author='Jina AI',
    author_email='hello@jina.ai',
    license='Apache 2.0',
    url='https://github.com/jina-ai/langchain-serve/',
    download_url='https://github.com/jina-ai/langchain-serve/tags',
    long_description=_long_description,
    long_description_content_type='text/markdown',
    zip_safe=False,
    setup_requires=['setuptools>=18.0', 'wheel'],
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'langchain-serve=lcserve.__main__:serve',
            'lc-serve=lcserve.__main__:serve',
            'lcserve=lcserve.__main__:serve',
        ],
    },
    extras_require={
        'test': [
            'pytest',
            'pytest-asyncio',
            'psutil',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
    ],
    project_urls={
        'Documentation': 'https://docs.jina.ai',
        'Source': 'https://github.com/jina-ai/now',
        'Tracker': 'https://github.com/jina-ai/now/issues',
    },
    keywords='jina langchain llm neural-network deep-learning data democratization',
)
